class TreeNode {
    constructor(value) {
      this.value = value;
      this.left = null;
      this.right = null;
    }
  }
  
  class BinaryTree {
    constructor() {
      this.root = null;
    }
  
    // Insert a node in the binary tree
    insert(value) {
      const newNode = new TreeNode(value);
      if (this.root === null) {
        this.root = newNode;
      } else {
        this.#insertNode(this.root, newNode);
      }
    }
  
    #insertNode(node, newNode) {
      if (newNode.value < node.value) {
        if (node.left === null) {
          node.left = newNode;
        } else {
          this.#insertNode(node.left, newNode);
        }
      } else {
        if (node.right === null) {
          node.right = newNode;
        } else {
          this.#insertNode(node.right, newNode);
        }
      }
    }
  
    // In-order traversal
    inOrder(node = this.root, result = []) {
      if (node !== null) {
        this.inOrder(node.left, result);
        result.push(node.value);
        this.inOrder(node.right, result);
      }
      return result;
    }
  
    // Pre-order traversal
    preOrder(node = this.root, result = []) {
      if (node !== null) {
        result.push(node.value);
        this.preOrder(node.left, result);
        this.preOrder(node.right, result);
      }
      return result;
    }
  
    // Post-order traversal
    postOrder(node = this.root, result = []) {
      if (node !== null) {
        this.postOrder(node.left, result);
        this.postOrder(node.right, result);
        result.push(node.value);
      }
      return result;
    }
  
    // Find the minimum value in the tree
    findMin() {
      if (!this.root) {
        return null;
      }
      let current = this.root;
      while (current.left !== null) {
        current = current.left;
      }
      return current.value;
    }
  
    // Find the maximum value in the tree
    findMax() {
      if (!this.root) {
        return null;
      }
      let current = this.root;
      while (current.right !== null) {
        current = current.right;
      }
      return current.value;
    }
  
    // Search for a value in the tree
    search(value) {
      return this.#searchNode(this.root, value);
    }
  
    #searchNode(node, value) {
      if (node === null) {
        return false;
      }
      if (value < node.value) {
        return this.#searchNode(node.left, value);
      } else if (value > node.value) {
        return this.#searchNode(node.right, value);
      } else {
        return true;
      }
    }
  }
  
  module.exports = BinaryTree;
  